// ScoresI is a simple base scores editor for Acacia / Infusion, written in Delphi 5.
// It is written by and copyright Doug Burbidge, 2013.

unit ScoresIMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, ComCtrls, ExtCtrls, Menus, Registry, Dialogs, Math, Buttons,
  Db, ZDataset, ZConnection, Spin;

type
  TMainForm = class(TForm)
    ScoreSetCombo: TComboBox;
    ZConnection: TZConnection;
    Label1: TLabel;
    Label2: TLabel;
    ShootsSpin: TSpinEdit;
    DestroysSpin: TSpinEdit;
    ChangeButton: TButton;
    NotSimpleLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ScoreSetComboChange(Sender: TObject);
    procedure ChangeButtonClick(Sender: TObject);
  private
    procedure ConnectDB;
    function  RegReadStringDef(R: TRegistry; Name, Default: string): string;
    function GetScoreSet: string;
  end;

implementation

{$R *.DFM}

// Free the passed-in object, and suppress any exception.
// This is used because with latest Firebird, ZEOS' query object throws an access violation when freed.
procedure FreeSquish(Obj: TObject);
begin
  try
    Obj.Free;
  finally
    // Squish.
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  ConnectDB;
end;

// read a string from the registry if it exists, otherwise return a default value
function TMainForm.RegReadStringDef(R: TRegistry; Name, Default: string): string;
begin
  if R.ValueExists(Name) and (R.GetDataType(Name) = rdString) then
    Result := R.ReadString(Name)
  else
    Result := Default;
end;

procedure TMainForm.ConnectDB;
var MyRegistry: TRegistry;
    DbPath: string;
    zquery: TZReadOnlyQuery;
begin
  ZConnection.Disconnect;

  MyRegistry := TRegistry.Create;  // open the Registry
  try
    MyRegistry.RootKey := HKEY_LOCAL_MACHINE;
    // Try to pull a path to the database server out of the registry.
    if MyRegistry.KeyExists('Software\PCMicro\Acacia') then begin
      MyRegistry.OpenKey('Software\PCMicro\Acacia', False);
      ZConnection.HostName := RegReadStringDef(MyRegistry, 'DBServerIP', '127.0.0.1');
      DbPath := RegReadStringDef(MyRegistry, 'DBPath', 'C:\Program Files\P&CMicros\Database\ZONESYSTEMS.GDB');
      MyRegistry.CloseKey;
    end else if MyRegistry.KeyExists('Software\DougWare\Torn') then begin
      MyRegistry.OpenKey('Software\DougWare\Torn', False);

      ZConnection.HostName := RegReadStringDef(MyRegistry, 'DBServerIP', '127.0.0.1');

      MyRegistry.CloseKey;
    end else
      DbPath := '';
  finally
    MyRegistry.Free;
  end;

  ZConnection.Protocol := 'interbase-5';
  ZConnection.Database := DbPath;
  ZConnection.User := 'sysdba';
  ZConnection.Password := 'masterkey';

  ZConnection.ReadOnly := True;

  zquery := TZReadOnlyQuery.Create(Self);
  try
    zquery.Connection := ZConnection;
    zquery.SQL.Add('SELECT ScoreSet_ID, Description FROM T2SCORESETCONFIG ORDER BY ScoreSet_ID');

    zquery.Open;
    while not zquery.EOF do begin
      ScoreSetCombo.Items.Add(IntToStr(zquery.FieldValues['ScoreSet_ID']) + ': ' + zquery.FieldValues['Description']);
      zquery.Next;
    end;
  finally
    FreeSquish(zquery);
  end;
end;

function TMainForm.GetScoreSet: string;
begin
  Result := ScoreSetCombo.Items[ScoreSetCombo.ItemIndex];
  Result := Copy(Result, 1, Pos(':', Result) - 1);
end;

procedure TMainForm.ScoreSetComboChange(Sender: TObject);
var zquery: TZReadOnlyQuery;
    shoots, destroys: Integer;
    simple: Boolean;
begin
  zquery := TZReadOnlyQuery.Create(Self);
  try
    zquery.Connection := ZConnection;
    zquery.SQL.Add('SELECT ShotBy_Location_1, ShotBy_Location_6 FROM T2SCORING ' +
                   'WHERE ScoreSet_ID = ' + GetScoreSet + ' AND ShotBy_Team = -1 AND (HitOn_Team = 4 OR HitOn_Team = 5 OR HitOn_Team = 6)');
    zquery.Open;
    simple := not zquery.EOF;
    if simple then begin
      shoots := zquery.FieldValues['ShotBy_Location_1'];
      destroys := zquery.FieldValues['ShotBy_Location_6'];

      zQuery.Next;
    end else begin
      shoots := 0;
      destroys := 0;
    end;

    while not zquery.EOF do begin
      if (shoots <> zquery.FieldValues['ShotBy_Location_1']) or
         (destroys <> zquery.FieldValues['ShotBy_Location_6']) then
        simple := False;

      zquery.Next;
    end;
  finally
    FreeSquish(zquery);
  end;

  ShootsSpin.Enabled := simple;
  DestroysSpin.Enabled := simple;
  ChangeButton.Enabled := simple;
  NotSimpleLabel.Visible := not simple;

  ShootsSpin.Value := shoots;
  DestroysSpin.Value := destroys;
end;

procedure TMainForm.ChangeButtonClick(Sender: TObject);
begin
    ZConnection.ExecuteDirect('UPDATE T2SCORING SET ShotBy_Location_1 = ' + IntToStr(ShootsSpin.Value) + ', ShotBy_Location_6 = ' + IntToStr(DestroysSpin.Value) +
                   ' WHERE ScoreSet_ID = ' + GetScoreSet + ' AND ShotBy_Team = -1 AND (HitOn_Team = 4 OR HitOn_Team = 5 OR HitOn_Team = 6)');
end;

end.
